function  [X,Y] = lusolv(A,B,row)
% [X,Y] = lusolv(A,B,row)
% The matrix A must be in factored LU form,
% and row is the required pivoting information.
% A  is the n x n matrix obtained from LUfact, input.
% B is any nx1 vector, input.
% row is the pivoting information from LUfact, input.
% X is the solution to AX=B,  output.
% Y is the solution to LY=PB, output.
% Forward substitution followed
% by back substitution is used.
[n n] = size(A);
Y = zeros(n,1);
Y(1) = B(row(1));
for k = 2:n,
  Y(k) = B(row(k)) - A(row(k),1:k-1)*Y(1:k-1);
end
X = zeros(n,1);
X(n) = Y(n)/A(row(n),n);
for k = n-1:-1:1,
  X(k) = (Y(k) - A(row(k),k+1:n)*X(k+1:n))/A(row(k),k);
end
