function  [lambda,V] = power(A,X,epsilon,max1,show)
% [lambda,V] = power(A,X,epsilon,max1,show)
% The power method is used to find the dominant eigenpair.
% A is an n x n matrix, input.
% X is the starting vector, input.
% epsilon is the tolerance, input.
% max1 is the maximum number of iterations, input.
% lambda is the dominant eigenvalue, output.
% V is the dominant eigenvector, output.
if nargin==4, show = 0; end
lambda = 0;
cnt = 0;
err = 1;
done = 0;
iterating = 1;
state = iterating;
while ((cnt<=max1) & (state==iterating))
  Y = A*X;               % Matrix multiplication
  [m j] = max(abs(Y));
  c1 = Y(j);             % Find "largest" element of Y.
  dc = abs(lambda-c1);
   Y = (1/c1)*Y;         % Perform scalar multiplication
  dv = norm(X-Y);
  err = max(dc,dv);
  X = Y;                 % Update vector  X
  lambda = c1;           % Update scalar lambda
  state = done;
  if (err>epsilon),      % Check for convergence
    state = iterating;
  end
  cnt = cnt+1;
  if show==1,
    home; if cnt==1, clc; end; 
    disp(['Power Method iteration No. ',int2str(cnt)]),disp(''),...
    disp('lambda = '),disp(lambda),...
    disp('Eigenvector = '),disp(X)
  end
end
V = X;
