echo off;
% NUMERICAL METHODS: MATLAB Programs, (c) John H. Mathews 1994
% To accompany the text:
% NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
% Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
% This free software is complements of the author.

% Algorithm 2.6 (Secant Method).
% Section	2.4,	Newton-Raphson and Secant Methods, Page 85
echo on; clc; format long; hold off; clear

% This program implements the secant method.

%	Define and store the function f(x) in the M-file  f.m  
% function y = f(x)
% y = x.^3 - 3.*x + 2;

delete f.m
diary f.m; disp('function y = f(x)');...
           disp('y = x.^3 - 3.*x + 2;');...
diary off;

% Remark. f.m and secant.m are used for Algorithm 2.6
f(0); % Test for file f.m
pause	% Press any key to see the graph y = f(x).

clc; clg
a = -2.5;
b = 2.5;
c = -5;
d = 5;
h = (b-a)/150;
X = a:h:b;
Y = f(X);
axis([a b c d]);...
plot(X,Y,'-g');...
hold on;...
plot([a b],[0 0],'b',[0 0],[c d],'b');...
xlabel('x');...
ylabel('y');...
title('Graph of y = f(x).');...
grid;...
axis;...
hold off;...
shg; pause	% Press any key to perform secant iteration.

clc;
%	Place the starting value in  p0  and  p1

%	Place the abscissa tolerance in  delta

%	Place the ordinate tolerance in  epsilon

%	Place the maximum number of iterations in  max1

p0 = -2.6;
p1 = -2.4;
delta = 1e-12;
epsilon = 1e-12;
max1  = 50;

[p1,y1,err,P] = secant('f',p0,p1,delta,epsilon,max1);

pause	% Press any key for the list of iterations.

clc; clg;
a = -2.7;
b = -1.9;
c = -10;
d = 2;
h = (b-a)/150;
X = a:h:b;
Y = f(X);
max1 = length(P);
clear Vx Vy
for i = 2:max1,
  k1 = 3*i-2;
  k2 = 3*i-1;
  k3 = 3*i;
  Vx(k1) = P(i);
  Vy(k1) = 0;
  Vx(k2) = P(i);
  Vy(k2) = f(P(i));
  Vx(k3) = P(i-1);
  Vy(k3) = f(P(i-1));
end
Z = zeros(1,length(P));
axis([a b c d]);...
plot(X,Y,'-g',Vx,Vy,'-r',P,Z,'or');...
hold on;...
plot([a b],[0 0],'b',[0 0],[c d],'b');...
xlabel('x');...
ylabel('y');...
title('Graphical analysis for the secant method.');...
grid;...
axis;...
hold off;...
shg; pause	% Press any key to continue.

J = 1:(max1);
Yp = f(P);
points = [J;P;Yp];
Mx1 = 'Iterations for the secant method.';
Mx2 = '     k                  p(k)               f(p(k))';
Mx3 = 'The solution is:';
Mx4 = 'The error estimate for p is   ';
clc,echo off, diary output,...
disp(''),disp(Mx1),disp(''),disp(Mx2),disp(points'),...
disp('Iteration converged with order 1.618 to the simple root.'),...
disp(''),disp(Mx3),disp(''),disp('p = '),...
disp(p1),disp(''),disp('f(p) = '),disp(y1),...
disp([Mx4,num2str(err)]),diary off,echo on

pause	% Press any key to perform secant iteration.

clc;
%	Place the starting value in  p0  and  p1

%	Place the abscissa tolerance in  delta

%	Place the ordinate tolerance in  epsilon

%	Place the maximum number of iterations in  max1

p0 = 1.6;
p1 = 1.4;
delta = 1e-12;
epsilon = 1e-12;
max1  = 30;

[p1,y1,err,P] = secant('f',p0,p1,delta,epsilon,max1);

pause	% Press any key for the list of iterations.

clg;
a = 0.95;
b = 1.6;
c = -0.1;
d = 1.2;
h = (b-a)/150;
X = a:h:b;
Y = f(X);
max1 = length(P);
clear Vx Vy
for i = 2:max1,
  k1 = 3*i-2;
  k2 = 3*i-1;
  k3 = 3*i;
  Vx(k1) = P(i);
  Vy(k1) = 0;
  Vx(k2) = P(i);
  Vy(k2) = f(P(i));
  Vx(k3) = P(i-1);
  Vy(k3) = f(P(i-1));
end
Z = zeros(1,length(P));
axis([a b c d]);...
plot(X,Y,'-g',Vx,Vy,'-r',P,Z,'or');...
hold on;...
plot([a b],[0 0],'b',[0 0],[c d],'b');...
xlabel('x');...
ylabel('y');...
title('Graphical analysis for the secant method.');...
grid;...
axis;...
hold off;...
shg; pause	% Press any key to continue.

J = 1:(max1);
Yp = f(P);  % User defined function.
points = [J;P;Yp];
clc,echo off, diary output,...
disp(''),disp(Mx1),disp(''),disp(Mx2),disp(points'),...
disp('Iteration is converging linearly to the double root.'),...
disp(''),disp(Mx3),disp(''),disp('p = '),...
disp(p1),disp(''),disp('f(p) = '),disp(y1),...
disp([Mx4,num2str(err)]),diary off,echo on

