echo off;
% NUMERICAL METHODS: MATLAB Programs, (c) John H. Mathews 1994
% To accompany the text:
% NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
% Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
% This free software is complements of the author.

% Algorithm 2.7 (Steffensen's Acceleration).
% Section	2.5,	Aitken's Process & Steffensen's & Muller's Methods, Page 96
echo on; clc; format long; hold off; clear
% This program implements the Steffensen method.

%	Define and store the functions f(x) and f'(x)

%	in the M-files  f.m  and  df.m, respectively.
% function y = f(x)
% y = x.^3 - 3.*x + 2;

% function y1 = df(x)
% y1 = 3*x.^2 - 3;

delete f.m
diary f.m; disp('function y = f(x)');...
           disp('y = x.^3 - 3.*x + 2;');...
diary off;

delete df.m
diary df.m; disp('function y1 = df(x)');...
            disp('y1 = 3*x.^2 - 3;');...
diary off;

% Remark. f.m, df.m and steff.m are used for Algorithm 2.7
f(0); df(0); % Test for files f.m, df.m
pause	% Press any key to see the graph y = f(x).

clc; clg;
%	Plot f(x) over the interval [a,b].

a = -3.0;
b = 3.0;
c = -10;
d = 10;
h = (b-a)/150;
X = a:h:b;
Y = f(X);
axis([a b c d]);...
plot(X,Y,'-g');...
hold on;...
plot([a b],[0 0],'b',[0 0],[c d],'b');...
xlabel('x');...
ylabel('y');...
title('Graph of y = f(x).');...
grid;...
axis;...
hold off;...
shg; pause	% Press any key to perform Steffensen iteration.

clc;
%	Place the starting value in  p0

%	Place the abscissa tolerance in  delta

%	Place the ordinate tolerance in  epsilon

%	Place the number of iterations in  max1

p0  = -2.4;
delta = 1e-12;
epsilon = 1e-12;
max1 = 5;

[p,yp,err,Q] = steff('f','df',p0,delta,epsilon,max1);

pause	% Press any key for the list of iterations.

clc; clg;
a = -2.42;
b = -1.97;
c = -5.0;
d = 0.5;
h = (b-a)/150;
X = a:h:b;
Y = f(X);
max1 = length(Q);
n0 = min(6,max1);
X0 = Q(1:n0);
Z0 = zeros(1,n0);
axis([a b c d]);...
plot(X,Y,'-g',X0,Z0,'or');...
hold on;...
plot([a b],[0 0],'b',[0 0],[c d],'b');...
xlabel('x');...
ylabel('y');...
title('Graphical analysis for Steffensen`s method.');...
grid;...
axis;...
hold off;...
shg; pause	% Press any key to continue.

J = 1:max1;
Yq = f(Q);
points = [J;Q;Yq];
Mx1 = 'Iterations for Steffensen`s method.';
Mx2 = ['     k                  p(k)               f(p(k))'];
Mx3 = 'The solution is:';
Mx4 = 'The error estimate for p is   ';
clc,echo off,diary output,
disp(''), disp(Mx1),disp(''), disp(Mx2), disp(points'),...
disp('Iteration converged quadratically to the root.'),...
disp(''),disp(Mx3),disp(''),disp('p = '),...
disp(p),disp(''),disp('f(p) = '),disp(yp),...
disp([Mx4,num2str(err)]),diary off,echo on

pause	% Press any key to perform Steffensen iteration.

clc;
%	Place the starting value in  p0

%	Place the abscissa tolerance in  delta

%	Place the ordinate tolerance in  epsilon

%	Place the number of iterations in  Max

p0  = 1.2;
delta = 1e-12;
epsilon = 1e-12;
max1 = 5;

[p,yp,err,Q] = steff('f','df',p0,delta,epsilon,max1);

pause	% Press any key for the list of iterations.

clc; clg;
a = 0.975;
b = 1.225;
c = -0.02;
d = 0.14;
h = (b-a)/150;
X = a:h:b;
Y = f(X);
max1 = length(Q);
n0 = min(6,max1);
X0 = Q(1:n0);
Z0 = zeros(1,n0);
axis([a b c d]);...
plot(X,Y,'-g',X0,Z0,'or');...
hold on;...
plot([a b],[0 0],'b',[0 0],[c d],'b');...
xlabel('x');...
ylabel('y');...
title('Graphical analysis for Steffensen`s method.');...
grid;...
axis;...
hold off;...
shg; pause	% Press any key to continue.

J = 1:max1;
Yq = f(Q);
points = [J;Q;Yq];
clc,echo off,diary output,
disp(''), disp(Mx1),disp(''), disp(Mx2), disp(points'),...
disp('Iteration converged quadratically to the root.'),...
disp(''),disp(Mx3),disp(''),disp('p = '),...
disp(p),disp(''),disp('f(p) = '),disp(yp),...
disp([Mx4,num2str(err)]),diary off,echo on

